<?php
class ControllerExtensionPaymentMorune extends Controller
{
	public function index()
	{
		return $this->load->view('extension/payment/morune');
	}

	public function confirm()
	{
		$this->load->language('extension/payment/morune');
		$this->load->model('checkout/order');
		$this->load->model('extension/payment/morune');

		$json = [];

		$secret = $this->config->get('payment_morune_secret');
		$shop_id = $this->config->get('payment_morune_shop_id');
		$additional = $this->config->get('payment_morune_additional');
		$services = $this->config->get('payment_morune_services');

		if (isset($this->session->data['payment_method']['code']) && $this->session->data['payment_method']['code'] == 'morune' && !empty($secret) && !empty($shop_id) && is_array($services) && !empty($services)) {

			$order_info = $this->model_checkout_order->getOrder($this->session->data['order_id']);

			if ($order_info && in_array($order_info['currency_code'], ['RUB', 'USD', 'EUR', 'UAH'])) {

				$failed_url = $this->url->link('extension/payment/morune/failed/' . $this->session->data['order_id']);
				$success_url = $this->url->link('extension/payment/morune/success/' . $this->session->data['order_id']);
				$hook_url = $this->url->link('extension/payment/morune/hook/' . $this->session->data['order_id']);

				$url = "https://api.morune.com/invoice/create";

				$headers = [
					"accept: application/json",
					"content-type: application/json",
					"x-api-key: {$secret}"
				];

				$invoiceData = [
					"amount" => $order_info['total'],
					"order_id" => (string) $this->session->data['order_id'], 
					"currency" => $order_info['currency_code'],
					"custom_fields" => ["order_id" => $this->session->data['order_id'], "key" => md5($order_info['date_added'])],
					"comment" => $order_info['store_name'] . ', ' . $this->language->get('text_order_number') . ' №' . $order_info['order_id'],
					"fail_url" => $failed_url,
					"success_url" => $success_url,
					"hook_url" => $hook_url,
					"shop_id" => $shop_id,
					"expire" => 300, 
					"include_service" => $services
				];
 
				$ch = curl_init();

				curl_setopt($ch, CURLOPT_URL, $url);
				curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
				curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
				curl_setopt($ch, CURLOPT_POST, true);
				curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($invoiceData));

				$response = curl_exec($ch);

				if (curl_errno($ch)) {

					$this->model_extension_payment_morune->log($this->session->data['order_id'], 'Ошибка cURL: ' . curl_error($ch));

				}

				$httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);

				curl_close($ch);

				if ($httpCode == 200) {

					$data = json_decode($response, true);

					if ($data['status'] == 200 && is_array($data['data']) && isset($data['data']['url']) && !empty($data['data']['url'])) {

						$this->model_checkout_order->addOrderHistory($this->session->data['order_id'], $this->config->get('payment_morune_order_status_id'));
						$this->model_extension_payment_morune->saveTransaction($this->session->data['order_id'], $data);
						$this->model_extension_payment_morune->log($this->session->data['order_id'], 'Redirect url получен успешно! Ответ сервера: ' . $response);

						$json['redirect'] = $data['data']['url'];

					} else {

						$this->model_extension_payment_morune->log($this->session->data['order_id'], 'Ошибка! Ответ сервера: ' . $response);

						$json['redirect'] = $this->url->link('extension/payment/morune/failed/' . $this->session->data['order_id']);

					}

				} else {

					$this->model_extension_payment_morune->log($this->session->data['order_id'], 'Ошибка! Код ответа сервера: ' . $httpCode . ' ' . $response);

					$json['redirect'] = $this->url->link('extension/payment/morune/failed/' . $this->session->data['order_id']);

				}

			} else {

				$this->model_extension_payment_morune->log($this->session->data['order_id'], 'Ошибка! Не найден заказ либо неверная валюта платежа!');

				$json['redirect'] = $this->url->link('extension/payment/morune/failed/' . $this->session->data['order_id']);

			}

		} else {

			$this->model_extension_payment_morune->log($this->session->data['order_id'], 'Ошибка! Неверный код либо пустой $secret или $shop_id либо не выбраны тарифы!');

			$json['redirect'] = $this->url->link('extension/payment/morune/failed/' . $this->session->data['order_id']);

		}

		$this->response->addHeader('Content-Type: application/json');
		$this->response->setOutput(json_encode($json));
	}

	public function failed()
	{

		$this->load->model('checkout/order');

		$order_id = 0;
		$url = $this->request->server['QUERY_STRING'];
		$url = explode('/', $url);
		$order_id = (int) array_pop($url);

		if ($order_id) {

			$this->model_checkout_order->addOrderHistory($order_id, $this->config->get('payment_morune_failed_order_status_id'));

		}

		$this->load->language('checkout/failure');

		if (isset($this->session->data['order_id'])) {

			$data['order_id'] = $this->session->data['order_id'];
			$this->cart->clear();

			unset($this->session->data['shipping_method']);
			unset($this->session->data['shipping_methods']);
			unset($this->session->data['payment_method']);
			unset($this->session->data['payment_methods']);
			unset($this->session->data['guest']);
			unset($this->session->data['comment']);
			unset($this->session->data['order_id']);
			unset($this->session->data['coupon']);
			unset($this->session->data['reward']);
			unset($this->session->data['voucher']);
			unset($this->session->data['vouchers']);
			unset($this->session->data['totals']);
		}

		$this->document->setTitle($this->language->get('heading_title'));

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_home'),
			'href' => $this->url->link('common/home')
		);

		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_basket'),
			'href' => $this->url->link('checkout/cart')
		);

		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_checkout'),
			'href' => $this->url->link('checkout/checkout', '', true)
		);

		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_failure'),
			'href' => $this->url->link('checkout/success')
		);

		$data['text_message'] = $this->language->get('text_message');

		$data['continue'] = $this->url->link('common/home');

		$data['column_left'] = $this->load->controller('common/column_left');
		$data['column_right'] = $this->load->controller('common/column_right');
		$data['content_top'] = $this->load->controller('common/content_top');
		$data['content_bottom'] = $this->load->controller('common/content_bottom');
		$data['footer'] = $this->load->controller('common/footer');
		$data['header'] = $this->load->controller('common/header');

		$this->response->setOutput($this->load->view('common/success', $data));

	}


	public function success()
	{

		$this->load->model('checkout/order');

		$order_id = 0;
		$url = $this->request->server['QUERY_STRING'];
		$url = explode('/', $url);
		$order_id = (int) array_pop($url);

		if ($order_id) {

			$this->model_checkout_order->addOrderHistory($order_id, $this->config->get('payment_morune_order_status_id'));

		}

		$this->load->language('checkout/success');

		if (isset($this->session->data['order_id'])) {

			$data['order_id'] = $this->session->data['order_id'];
			$this->cart->clear();

			unset($this->session->data['shipping_method']);
			unset($this->session->data['shipping_methods']);
			unset($this->session->data['payment_method']);
			unset($this->session->data['payment_methods']);
			unset($this->session->data['guest']);
			unset($this->session->data['comment']);
			unset($this->session->data['order_id']);
			unset($this->session->data['coupon']);
			unset($this->session->data['reward']);
			unset($this->session->data['voucher']);
			unset($this->session->data['vouchers']);
			unset($this->session->data['totals']);
		}

		$this->document->setTitle($this->language->get('heading_title'));

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_home'),
			'href' => $this->url->link('common/home')
		);

		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_basket'),
			'href' => $this->url->link('checkout/cart')
		);

		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_checkout'),
			'href' => $this->url->link('checkout/checkout', '', true)
		);

		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_success'),
			'href' => $this->url->link('checkout/success')
		);

		if ($this->customer->isLogged()) {
			$data['text_message'] = sprintf($this->language->get('text_customer'), $this->url->link('account/account', '', true), $this->url->link('account/order', '', true), $this->url->link('account/download', '', true), $this->url->link('information/contact'));
		} else {
			$data['text_message'] = sprintf($this->language->get('text_guest'), $this->url->link('information/contact'));
		}

		$data['continue'] = $this->url->link('common/home');

		$data['column_left'] = $this->load->controller('common/column_left');
		$data['column_right'] = $this->load->controller('common/column_right');
		$data['content_top'] = $this->load->controller('common/content_top');
		$data['content_bottom'] = $this->load->controller('common/content_bottom');
		$data['footer'] = $this->load->controller('common/footer');
		$data['header'] = $this->load->controller('common/header');

		$this->response->setOutput($this->load->view('common/success', $data));

	}

	public function hook()
	{

		if ($_SERVER['REQUEST_METHOD'] !== 'POST') {
               exit;
		}
		
		$this->load->model('extension/payment/morune');
		$this->load->model('checkout/order');

		$order_id = 0;
		$url = $this->request->server['QUERY_STRING'];
		$url = explode('/', $url);
		$order_id = (int) array_pop($url);
		$order_info = $this->model_checkout_order->getOrder($order_id);

		$secret = $this->config->get('payment_morune_secret');
		$shop_id = $this->config->get('payment_morune_shop_id');
		$additional = $this->config->get('payment_morune_additional');

		//$headerSignature = $this->request->server['x-api-sha256-signature'];
		$headerSignature = $_SERVER['HTTP_X_API_SHA256_SIGNATURE'] ?? '';

		if(!$headerSignature) {
			  exit;
		}

		$body = file_get_contents('php://input');
		$data = json_decode($body, true);

		if(empty($data)) {
			exit;
		}
 
		if ($this->checkSignature($body, $headerSignature, $additional) && $order_info) {
 
			$status = $data['status'];
			$comment = 'Morune transaction id: ' . $data['invoice_id'];
		 
			$status = [

				'success' =>   $this->config->get('payment_morune_success_order_status_id'),
				'fail' =>  $this->config->get('payment_morune_failed_order_status_id'),
				'expired' =>  $this->config->get('payment_morune_failed_order_status_id'),
				'refund' => $this->config->get('payment_morune_return_order_status_id'),

			]; 
 
			$this->model_checkout_order->addOrderHistory($order_id, (int)$status[$data['status']], $comment, $notify = true, $override = false);
			$this->model_extension_payment_morune->updateTransaction($order_id, $data);
			$this->model_extension_payment_morune->log($order_id, 'OK x-api-sha256-signature!' . $body);
  
		} else {

			$this->model_extension_payment_morune->log($order_id, 'Ошибка! Неверная x-api-sha256-signature или $order_id!' . $body);

		}

	}
 

	private function checkSignature( $hookJson, $headerSignature, $secretKey) 
	{
		$hookArr = json_decode($hookJson, true);
		ksort($hookArr);
		$hookJsonSorted = json_encode($hookArr);
		$calculatedSignature = hash_hmac('sha256', $hookJsonSorted, $secretKey);
		return hash_equals($headerSignature, $calculatedSignature);
	}

}
